module RightsizingHelpers
  @generated_response_path = 'lib/support/recommendations/generated_recommendations.json'
  @generated_list_of_efficiency_targets = 'lib/support/recommendations/response.json'
  @default_eff_target_avg = 'crn:0:rightsizing-framework/AWS_EC2_EfficiencyTarget:system_avg'
  @default_eff_target_max = 'crn:0:rightsizing-framework/AWS_EC2_EfficiencyTarget:system_max'

  # Method to get the args for rightsizing recommendations
  def get_args_for_get_rightsizing_recommendations(efficiency_target_id, evaluation_duration)
    {
      request: {
        assetType: 'AWS_EC2',
        efficiencyTarget: efficiency_target_id,
        evaluationDuration: evaluation_duration
      },
      sortRules: [
        {
          field: 'cpu',
          direction: 'ASC'
        }
      ]
    }
  end

  # Method to get the rightsizing efficiency target ID
  # @param args - Arguments to be passed for getting the list
  def get_efficiency_target_id(args = {})
    response = create_rightsizing_efficiency_target(args)
    response[:efficiencyTarget][:id]
  end

  # Method to get the list of all the efficiency targets' IDs
  def get_list_of_efficiency_target_ids
    ids = Set.new
    response = get_rightsizing_efficiency_targets(null)
    response[:edges].each do |efficiency_targets_array|
      ids.add(efficiency_targets_array[:id])
    end
    ids
  end

  # Method to get the rightsizing recommendations summary
  # WIP
  def get_rightsizing_recommendations_summary
    efficiency_score = get_rightsizing_utilization_status_summary[1] / total_instances * 100
    projected_efficiency_score = 100
    monthly_cost = sum_of_the_costs_of_instances
    projected_monthly_cost = sum_of_the_monthly_costs_of_all_the_best_fit_instances
    projected_monthly_difference = monthly_cost - projected_monthly_cost
    [efficiency_score, projected_efficiency_score, monthly_cost, projected_monthly_cost, projected_monthly_difference]
  end

  # Method to compare the recommendations generated against the expected
  def compare_recommendations(instance_id, evaluation_duration)
    expected_response_path = "lib/support/recommendations/expected_recommendations_#{evaluation_duration}.json"
    f1 = File.read(@generated_response_path)
    f2 = File.read(expected_response_path)
    generated = JSON.parse(f1)
    expected = JSON.parse(f2)
    options_generated = {}
    options_expected = {}
    generated['edges'].each do |node|
      options_generated = node['options'] if node['targetAsset']['id'].eql?(instance_id)
    end
    expected['edges'].each do |node|
      options_expected = node['options'] if node['targetAsset']['id'].eql?(instance_id)
    end
    diff = Hashdiff.diff(options_generated, options_expected)
    raise(' recommendations are not matching') unless diff.nil?
  end

  # Method to calculate the efficiency score as per the mock file
  def get_efficiency_score_calculated
    on_target = get_rightsizing_utilization_status_summary_calculated[0]
    total_instances = get_total_instances
    on_target / total_instances * 100
  end

  # Method to calculate the projected efficiency score as per the mock file
  def get_projected_efficiency_score_calculated
    on_target = get_rightsizing_utilization_status_summary_calculated[0]
    total_instances = get_total_instances
    on_target / total_instances * 100
  end

  # Method to get the total instances for a particular customer using the sql query
  # As of now its returning 181 - this count is from redshirt account
  def get_total_instances
    # TODO: add the sql query here to call the same
    181
  end

  # Method to calculate the rightsizing utilization status summary from the mock file
  # ToDo: Currently hardcoded value to be changed to the ones being read from the mock csv file
  def get_rightsizing_utilization_status_summary_calculated
    on_target = 2
    under_target = 2
    over_target = 2
    notEnoughData = 2
    unknown = 2
    [on_target, under_target, over_target, notEnoughData, unknown]
  end

  # Method to get the efficiency score from the API
  def get_efficiency_score_from_API
    f1 = File.read(@generated_response_path)
    generated = JSON.parse(f1)
    generated['summary']['efficiencyScore']
  end

  # Method to calculate the projected efficiency score from API
  def get_projected_efficiency_score_from_API
    f1 = File.read(@generated_response_path)
    generated = JSON.parse(f1)
    generated['summary']['projectedEfficiencyScore']
  end

  # Method to validate the efficiency score
  def validate_efficiency_score
    raise('Efficiency score is not matching') unless get_efficiency_score_calculated == get_efficiency_score_from_API
  end

  # Method to validate the projected efficiency score
  def validate_projected_efficiency_score
    unless get_projected_efficiency_score_calculated == get_projected_efficiency_score_from_API
      raise('Projected efficiency score is not matching')
    end
  end

  # Method to get the rightsizing utilization status summary from API
  def get_rightsizing_utilization_status_summary_from_API
    f1 = File.read(@generated_response_path)
    generated = JSON.parse(f1)
    on_target = generated['summary']['utilization']['onTarget']
    under_target = generated['summary']['utilization']['underTarget']
    over_target = generated['summary']['utilization']['overTarget']
    notEnoughData = generated['summary']['utilization']['notEnoughData']
    unknown = generated['summary']['utilization']['unknown']
    [on_target, under_target, over_target, notEnoughData, unknown]
  end

  # Method to get the validate the rightsizing status summary
  def validate_rightsizing_uitilization_status_summary
    diff = Hashdiff(get_rightsizing_utilization_status_summary_from_API,
                    get_rightsizing_utilization_status_summary_calculated)
    raise('Rightsizing utilization status summary is not matching') if diff.nil?
  end

  # Method to validate the rightsizing recommendations summary
  def validate_rightsizing_recommendations_summary
    validate_efficiency_score
    validate_projected_efficiency_score
    validate_rightsizing_uitilization_status_summary
  end

  def validate_get_list_of_efficiency_target_API(response, id)
    response_ids = get_efficiency_ids(response)
    unless response_ids.include?(id)
      raise("Created IDs not present in the list, created ET is #{id} and the response from get ETs is #{response_ids}")
    end
  end

  def get_total_number_of_efficiency_targets(response)
    response['edges'].size
  end

  def get_efficiency_ids(response)
    response_ids = []
    response_edges = response[:edges]
    response_edges.each do |item|
      response_ids.push(item[:node][:id])
    end
    response_ids.sort
  end

  def validate_only_default_efficiency_targets_present(response)
    default_eff_targets_number = get_total_number_of_efficiency_targets(response)
    ids_list = [@default_eff_target_max, @default_eff_target_avg]
    response_ids = get_efficiency_ids(response)
    unless default_eff_targets_number == 2 && ids_list.eql?(response_ids)
      raise('Default efficiency target IDs not present')
    end
  end

  # Method to delete the efficiency target by passing the efficiency target ID as the param
  def delete_ET(et_id)
    variables = {
      id: et_id
    }
    delete(variables)
  end
end
