# Test cases related to updating/deleting of rightsizing efficiency targets

require 'spec_helper'

module RightsizingFrameworkTest
  describe 'RightsizingFramework updateRightsizingEfficiencyTarget' do
    include_context 'request metadata'

    # Test -1 : Validate if a user is able to update the efficiency target
    it 'Validate updating the efficiency target' do
      random_val = Random.new.rand
      metrics = {
        metricName: 'avg_cpu',
        aggregation: 'AVG',
        lowerRange: 30,
        upperRange: 70
      }
      args = {
        assetType: 'AWS_EC2',
        metrics: metrics,
        name: "Test_Delete_ET#{random_val}"
      }
      response = create_rightsizing_efficiency_target(args)
      id = response[:efficiencyTarget][:id]
      updated_name = "Updated_efficiency_target#{random_val}"
      variables = {
        id: id,
        name: updated_name,
        metrics: [
          {
            metricName: 'avg_cpu',
            aggregation: 'AVG',
            lowerRange: 20.0,
            upperRange: 80.0
          }
        ]
      }
      response_update = update_rightsizing_efficiency_target(variables)
      is_ids_same = response_update[:efficiencyTarget][:id].eql?(id)
      is_ET_name_same = response_update[:efficiencyTarget][:name].eql?(updated_name)
      is_lowerRange_same = response_update[:efficiencyTarget][:metrics][0][:lowerRange].eql?(variables[:metrics][0][:lowerRange])
      is_upperRange_same = response_update[:efficiencyTarget][:metrics][0][:upperRange].eql?(variables[:metrics][0][:upperRange])
      if !is_ids_same && is_ET_name_same && is_lowerRange_same && is_upperRange_same
        raise('Updating of the efficiency target failed')
      end

      delete_ET(id)
    end

    # Test case to validate if a user is able to update the efficiency target with invalid metrics
    it 'Validate updating the efficiency target with invalid metrics', monitoring: true do
      random_val = Random.new.rand
      metrics = {
        metricName: 'avg_cpu',
        aggregation: 'AVG',
        lowerRange: 30,
        upperRange: 70
      }
      args = {
        assetType: 'AZURE_VM',
        metrics: metrics,
        name: "Test_Update_ET#{random_val}"
      }
      response = create_rightsizing_efficiency_target(args)
      id = response[:efficiencyTarget][:id]
      updated_name = "Updated_efficiency_target#{random_val}"
      updated_metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'MAX',
          lowerRange: 20.0,
          upperRange: 80.0
        }
      ]
      wrap_request do
        variables = {
          input: {
            id: id,
            metrics: updated_metrics,
            name: updated_name
          }
        }
        graphql url(service: service),
                format_request(
                  :updateRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        raise('expected graphql error return, but no errors') unless
          json_body.key?(:errors)
      end
      delete_ET(id)
    end

    # Test case to delete the efficiency target
    it 'Deleting the efficiency target' do
      random_val = Random.new.rand
      metrics = {
        metricName: 'avg_cpu',
        aggregation: 'AVG',
        lowerRange: 30,
        upperRange: 70
      }
      args = {
        assetType: 'AWS_RDS',
        metrics: metrics,
        name: "Test_Delete_ET#{random_val}"
      }
      response = create_rightsizing_efficiency_target(args)
      id = response[:efficiencyTarget][:id]
      variables = {
        id: id
      }
      response_delete = delete(variables)
      raise('Efficiency target could not be deleted') unless response_delete.eql?(true)
    end

    # Negative scenario - Test case to delete the efficiency target which does not exist
    assetType = Array.[]('AWS_EC2', 'AWS_RDS','AWS_EBS')
    assetType.each do |asset|
    it 'E: Deleting the efficiency target which does not exist', monitoring: true do
      variables = {
        id: 'crn:0:rightsizing-framework/'+asset+'_EfficiencyTarget/2e8-c534-4f02-977e-485703fc8e02'
      }
      graphql url(service: service),
              format_request(
                :delete,
                service: service
              ),
              variables, headers
      raise('expected graphql error return, but no errors') unless
        json_body.key?(:errors)
        end
    end

    # Negative scenario - Test case to validate if a user is able to update the efficiency target which doesn't exist
    assetType = Array.[]('AWS_EC2', 'AWS_RDS','AWS_EBS')
    assetType.each do |asset|
    it 'E:Validate updating the efficiency target which does not exist', monitoring: true do
      id = 'crn:0:rightsizing-framework/AWS_EC2_EfficiencyTarget/8-c534-4f02-977e-485703fc8e02',
           updated_name = "Updated_efficiency_target#{Random.new}"
      updated_metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'MAX',
          lowerRange: 20.0,
          upperRange: 80.0
        }
      ]
      wrap_request do
        variables = {
          input: {
            id: id,
            metrics: updated_metrics,
            name: updated_name
          }
        }
        graphql url(service: service),
                format_request(
                  :updateRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        raise('expected graphql error return, but no errors') unless
          json_body.key?(:errors)
      end
      end
    end

    # Test case to delete the system defined efficiency target
    assetType = Array.[]('AWS_EC2', 'AWS_RDS','AWS_EBS')
    assetType.each do |asset|
    it 'E:Validate user is not able to delete system defined efficiency target', monitoring: true do
      id = 'crn:0:rightsizing/'+asset+'_EfficiencyTarget:system_max'
      variables = {
        id: id
      }
      graphql url(service: service),
              format_request(
                :delete,
                service: service
              ),
              variables, headers
      raise('expected graphql error return, but no errors') unless
        json_body.key?(:errors)
        end
    end

    # Test case to delete all the efficiency targets except system defined ones
    # ToDo: Add this TC as part of cleanup
    assetType = Array.[]('AWS_EC2', 'AWS_RDS','AZURE_VM','AWS_EBS')
    assetType.each do |asset|
    xit 'E:Validate user is not able to delete system defined efficiency target but delete all others', monitoring: true do
      id_1 = 'crn:0:rightsizing/'+asset+'_EfficiencyTarget:system_max'
      id_2 = 'crn:0:rightsizing/'+asset+'_EfficiencyTarget:system_avg'
      variables = {
      assetType: asset
      }
      json_response = rightsizing_efficiency_targets(variables)
      edges = json_response[:edges]
      ids = []
      edges.each do |edge|
        ids.push(edge[:node][:id])
      end
      ids.each do |id|
        next unless !id.eql?(id_1) && !id.eql?(id_2)

        variables = {
          id: id
        }
        delete(variables)
      end
      end
    end
  end
end
